* Nametable name scanner
*
* pjwitte Jan 2oo2
* V0.02a        - No translation table version
* V0.03a        - bug fix. Now works in Qdos
* V0.04         - added SMSQ/E mnemonics, no functional change
* V0.05         - bug fix (and linked to correct library!)
* V0.06         - scan local and/or global name table
* V0.07, pjw, 2024 Jul 29, changed default to scan local and global
*
* Compatible with SMSQ/E, Qdos and compiled
* Turbo: With flag 1 and -1 will crash (but no flag and flag = 0 should be ok)

        section code

        include dev8_keys_sbasic
        include dev8_keys_jcb
        include dev8_keys_qdos_sms
        include dev8_keys_sys


        xdef FINDNAME

        xref getpars
        xref strequnt

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Given a function or procedure name, FINDNAME% searches nametable for a
* match
*
* type% = FINDNAME%(name$ [, flag%])
*
* where type% = 0 => no match found
*               1 => procedure found
*               2 => function found
*
*       name$ = upper or lower case (complete) name to search for
*
*       flag% =  0 => first check local then, if not found, global (default)
*                1 => check local name table only
*               -1 => check global name table only
*
* Note on flag: If used from job #0, local and global are the same, so
*               only global is scanned.


FINDNAME
        moveq #$14,d0           get 1 string
        bsr getpars
        bne.s exit

        moveq #0,d6             flag default (job #0 only)
        cmpa.l a3,a5            any more pars?
        beq.s one                no, only the one

        moveq #$10,d0           get 1 int
        bsr getpars
        bne.s exit

        move.w 0(a6,a1.l),d6    d6 = flag
        addq.l #2,a1
        move.l a1,sb_arthp(a6)  tidy int off stack

one
        moveq #sms.info,d0
        trap #do.smsq
        movea.l a0,a4           a4-> sys vars for safe keeping
        tst.l d1                job #0?
        bne.s local

        bsr.s find              all become local for job #0
        bra.s return

local
        tst.w d6                only global wanted?
        bmi.s global

        bsr.s find              local (and possibly global)
        bne.s return            found locally

        tst.w d6                not found, try global?
        beq.s global             yes

        bra.s return             no, final answer: not found

global
        move.w sr,d5            flag supervisor mode
        trap #0                 enter supervisor mode

        movem.l a1/a6,-(sp)     save some regs

        adda.l a6,a1            make stack pointer absolute

        move.l sys_jbtb(a4),a6  -> job table (a4 -> sys vars)
        lea.l (a6),a6           ->> JCB
        movea.l (a6),a6         JCB#0
        move.l jcb_a6(a6),a6    a6 = job 0's a6!

        suba.l a6,a1            make stkptr relative to new a6
        bsr.s find

        movem.l (sp)+,a1/a6     restore ri and base ptr

        move.w d5,sr            exit svm

return
        moveq #1,d2
        add.w 0(a6,a1.l),d2
        bclr #0,d2              make even
        add.l d2,sb_arthp(a6)   reset stack, use len for return int
        movea.l sb_arthp(a6),a1

        move.w d3,0(a6,a1.l)
        moveq #ar.int,d4        ret integer
        moveq #0,d0

exit
        rts

find
        moveq #0,d4             prime len1

        move.l sb_nmtbb(a6),a2  a2 -> nametable base(rel a6)
        bra.s scan              returns d0 = nf, proc, fn

* * * * Main Loop * * * * *

next_loop
        addq.l #8,a2            next entry
        cmp.l sb_nmtbp(a6),a2    unless end (should never be on first round)
        bge.s not_found

scan
        move.w nt_usetp(a6,a2.l),d7 = type

* Filter out types

        cmpi.w #$800,d7         m/c proc or fn?
        bcs.s next_loop         dont bother with these

search
        move.w nt_name(a6,a2.l),d2 offset to name rel names area

        move.l sb_nmlsb(a6),a0  name list base
        adda.w d2,a0            offset to name rel a6

        bsr strequnt            strings same?
        bne.s next_loop          not

        cmpi.w #$800,d7         found!
        bhi.s retfn

        moveq #1,d3             flag proc
        rts
retfn
        moveq #2,d3             flag fn
        rts

not_found
        moveq #0,d3             not found
        rts

*
        end
